% Numerical Implementation for  
% 'X-Ray CT Image Reconstruction via Wavelet Frame Based Regularization and
% Radon Domain Inpainting'  Bin Dong, Jia Li, Zuowei Shen 
% a demo with #projection = 30
%%%%% close all;
close all
clc
rand('seed',2015);
randn('seed',2015);

%% Initialization
constant;

load CTimage;
dataTruth = Img;
NPROJ = 15; % #projection 

% generate projection matrix based on Sidden's Algorithm
proj2 = []; % augmented projection matrix 
for i = 1 : NPROJ*2,
    proj2 = [proj2; projMatrix2D(L1,L2, N, PI*i/2/NPROJ, L, ND)];
end
proj = []; % original projection matrix
for i = 1 : NPROJ,
    proj = [proj; projMatrix2D(L1,L2, N, PI*i/NPROJ, L, ND)];
end

% generate observed signal 
bvec = reshape(dataTruth,N*N,1);
bvec4 = proj2*bvec;
bvec = proj*bvec;

figure;
bvec2 = reshape(bvec,[ND,NPROJ]);
Noise = max(bvec2(:))/300.0*randn(size(bvec2));
bvec5 = bvec2;
bvec2 = bvec2 + Noise; % add noise 
bvec = reshape(bvec2,ND*NPROJ,1);
bvec4 = reshape(bvec4,[ND,NPROJ*2]);
imshow(bvec2,[]);


maxit = 60;
maxcgit = 50;
ind = ones(size(bvec));


% generate initialized $u$ and $f$ with wavelet model
lambda = 1.4; 
mu = 1000;
tol = 0.00001;
[dataRef, ~, w, q] = L2data_frame(bvec, proj, ind, lambda, mu,tol, maxit, maxcgit,N,dataTruth);

display(corr(dataTruth,dataRef));%display the relative error and correlation
display(epss(dataTruth,dataRef));

bvec3 = proj2*reshape(dataRef,N*N,1); % interpolated signal $f$
bvec3 = reshape(bvec3,ND,NPROJ*2);


%% Wavelet frame based and Inpainting Model

lambda = 0.7; 
mu = 1000;
maxit = 1000;
maxcgit = 50;
ind = ones(ND*NPROJ*2,1);
tol = 0.000001;
lambda2 = 8.0e-09;
mu2 = 0.00002;
lambda3 = 1;
maxcgit2 = 50;

tic;
    dataRes = L2data_framef(bvec2,bvec3, proj2, ind, lambda, mu, tol, maxit, maxcgit,N,dataTruth,bvec4,ND,NPROJ,lambda2, lambda3,mu2, maxcgit2,w,q,dataRef);
toc; 

display(corr(dataTruth,dataRes));
display(epss(dataTruth,dataRes));



save ori_res15 dataRef dataRes;
   

