%consider an image deburring problem with a known blurring kernal
%L2 data fidelity term + l_1 norm frame cofficients as regularization
function [u Res w q]= L2data_frame(Bn, H, ind, lambda, mu, tol, maxit, maxcgit,N,dataTruth)
%INPUT:
    %Bn -- Blurred noisy image
    %H -- blurring kernal
    %ind -- index of pixels used for fitting
    %lambda -- regularization parameter
    %mu -- parameter for split Bregman iteration
    %tol -- parameter for split Bregman iteration
    %maxit -- maximum iteration
    %maxcgit -- maximun number of CG iterations

%OUTPUT:
    %u -- deblurred image

    
%========================
%   framelet transform
%========================
order = 1; dlevel = 1; 
lowpass = kron([1/4 1/2 1/4]',[1/4 1/2 1/4]);
W = @(z) swt2(z, order, dlevel);
WT = @(z) iswt2(z, order, dlevel);

posi=ones(N,N);
posi2=ones(N,N);
% for i=1:N
%     for j=1:N
%         if((i-N/2-0.5)^2+(j-N/2-0.5)^2>153^2)
%             posi(i,j)=0;
%         end;
%     end;
% end;
for i=1:N
    for j=1:N
        if((i-N/2-0.5)^2+(j-N/2-0.5)^2>125^2)
            posi2(i,j)=0;
        end;        
    end;
end;
% posi2(:,1:50)=0;
% posi2(:,420:512)=0;
posi2(:,1:25)=0;
posi2(:,210:256)=0;

%==============================
%   split Bregman iteration
%==============================
%initialization
Res=zeros(maxit,2);
u = zeros(N,N);
w = W(u); q = w; %3D matrix to store frame coefficients

%iteration 
error = 10*tol;
it = 1; 
% while (error > tol && it <= maxit)
while (it <= maxit)
    %solve a linear system by the conjugate gradient method
    %an efficient alternative: diagonalize the matrix by DCT 
    tmp = ind.*Bn;
    %rhs = imfilter(tmp, H, 'symmetric','conv');
    rhs = H'*tmp;
    rhs=reshape(rhs,N,N);
%     rhs = conj_imfilter(tmp, H);
    w(:,:,1)=0; 
    q(:,:,1)=0;
    rhs = rhs+mu*WT(w-q);
    u = FrameL2_CG(H, ind, mu, lowpass, rhs, maxcgit,N);
    
    %soft-thresholding
    Wu = W(u);
    w = softthresh(Wu+q, lambda/mu,dlevel,posi2);
%     w = softthresh(Wu+q, lambda/mu);
%     w=Wu+q;
        
    %update residual
    ev1 = (Wu-w);
    q = q+ev1;
    
    %error
    ev1(:,:,1)=0;
    it = it+1;
    error = sqrt(sum(ev1(:).^2));
    fprintf('at iteration %d, residual norm is %f\n', it, error);
    u(find(u<0.0))=0.0;
%     u(posi2==0)=0.0;
    display(epss(dataTruth,u));
    display(corr(dataTruth,u));
%     Res(it,1)=epss(dataTruth(posi==1),u(posi==1));    
%     Res(it,2)=corr(dataTruth(posi==1),u(posi==1));
%     uuu=u;
%     uuu(posi2==0)=0;
    imshow(u,[]);
    drawnow;
%     if(it==20 || it==50)
%         figure;
%         imshow(u,[]);
%     end;
end
% u(posi2==0)=0.0;
% u(posi==0)=0.0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  the followings are auxiliary functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%conjugate gradient to the linear system (H'*H+mu(I-A'*A))x = rhs
%because A'A+D1'*D1+....+Dr'*Dr=I
function x = FrameL2_CG(H, ind, mu, lowpass, rhs, maxcgit,N)

x = zeros(size(rhs));
r = rhs;
d = rhs;
rtr = r(:)'*r(:);
niters = 0;
while (rtr > 1e-10 && niters < maxcgit)
    niters = niters+1;
    %compute Ad
    tmp=H*reshape(d,N*N,1);
%     tmp = imfilter(d, H, 'symmetric', 'conv');
    tmp = ind.*tmp;
%     tmp = imfilter(tmp, H, 'symmetric', 'conv');
%     tmp = conj_imfilter(tmp, H);
    tmp=reshape(H'*tmp,N,N);
    Ad = tmp+mu*d;
    tmp = imfilter(d, lowpass, 'symmetric', 'conv');
    tmp = imfilter(tmp, lowpass, 'symmetric', 'conv');
    Ad = Ad-mu*tmp;
    %compute Ad
    alpha = rtr/(d(:)'*Ad(:));
    x = x + alpha*d;
    r = r - alpha*Ad;
    rtrold = rtr;
    rtr = r(:)'*r(:);
    beta = rtr/rtrold;
    d = r + beta*d;
end


% soft-thresholding
function ds1 = softthresh(ds,t,level,posi)
ds1=ds;
[a b c] = size(ds);
d=(c-1)/level;
dn=double(zeros(a,b));
tp=zeros(1,c);
for l = 1:level
    for i = 1:a
        for j = 1:b
            tp(:)=ds(i,j,:);
            dn(i,j) = norm(tp(2+(l-1)*d:d*l+1),2);
        end;
    end;
    res = (abs(dn) - t*sqrt(d)*2/(2^l));
    res = (res + abs(res))/2;
    for k = 2+(l-1)*d:d*l+1
      ds1(:,:,k) = ds(:,:,k).*res./(dn+1e-20);
    end;
end;
% for i = 1:a
%     for j = 1:b
%         if(posi(i,j)==0)
%             ds1(i,j,:)=ds(i,j,:);
%         end;
%     end;
% end;
% ds1(posi==0)=ds(posi==0);

% function ds1 = softthresh(ds,t)
% 
% res = (abs(ds) - t);
% res = (res + abs(res))/2;
% ds1 = sign(ds).*res;


%compute D1'*u(D2'*u) by imfilter, where D1 and D2 correspond to 
%non-symmetric filter with symmetric boundary condition
function ru = conj_imfilter(u, H)

ru = imfilter(u, fliplr(flipud(H)), 'conv');

% function ru = conj_imfilter(u, fil)
% 
% sym_part = imfilter(u, fil, 'symmetric', 'conv')-imfilter(u,fil, 0, 'conv');
% nonsys_part = imfilter(u, flipud(fliplr(fil)), 0, 'conv');
% ru = sym_part+nonsym_part;
%=======================================================