% Numerical Implementation for  
% 'CT Image Reconstruction by Spatial-Radon Domain Data-Driven Tight Frame
% Regularization' Ruohan Zhan, Bin Dong
% a demo with #projection = 15
%%%%% close all;
close all
clc

rand('seed',2015);
randn('seed',2015);
%% Initialization
constant;

load CTimage;
dataTruth = Img;
NPROJ = 15; % #projection 

% generate projection matrix based on Sidden's Algorithm
proj2 = []; % augmented projection matrix 
for i = 1 : NPROJ*2,
    proj2 = [proj2; projMatrix2D(L1,L2, N, PI*i/2/NPROJ, L, ND)];
end
proj = []; % original projection matrix
for i = 1 : NPROJ,
    proj = [proj; projMatrix2D(L1,L2, N, PI*i/NPROJ, L, ND)];
end

% generate observed signal 
bvec = reshape(dataTruth,N*N,1);
bvec4 = proj2*bvec;
bvec = proj*bvec;

figure;
bvec2 = reshape(bvec,[ND,NPROJ]);
Noise = max(bvec2(:))/300.0*randn(size(bvec2));
bvec5 = bvec2;
bvec2 = bvec2 + Noise; % add noise 
bvec = reshape(bvec2,ND*NPROJ,1);
bvec4 = reshape(bvec4,[ND,NPROJ*2]);
imshow(bvec2,[]);


maxit = 60;
maxcgit = 50;
ind = ones(size(bvec));


% generate initialized $u$ and $f$ with wavelet model
lambda = 1.4; 
mu = 1000;
tol = 0.00001;
% Use analysis based model to generate initial u
[dataRef] = L2data_frame(bvec, proj, ind, lambda, mu,tol, maxit, maxcgit,N,dataTruth); 

display(corr(dataTruth,dataRef));%For display the relative error and correlation
display(epss(dataTruth,dataRef));

bvec3 = proj2*reshape(dataRef,N*N,1); % interpolated signal $f$
bvec3 = reshape(bvec3,ND,NPROJ*2);


%% SRD_DDTF Model

lambda = 0.03; 
mu = 1000;
maxit = 1000;
maxcgit = 50;
ind = ones(ND*NPROJ*2,1);
lambda2 = 9.0e-08;
mu2 = 0.00002;
lambda3 = 1;
maxcgit2 = 50;

tic;
    dataRes = data_framef2(dataRef,bvec3,bvec2,bvec4,proj2,lambda3,lambda,lambda2,ND,NPROJ,dataTruth,maxit,maxcgit);
toc; 

display(corr(dataTruth,dataRes));
display(epss(dataTruth,dataRes));


save SRDDDTF_res15 dataRef dataRes;
   

