% SRD_DDTF model for CT image reconstruction
function [DataRes,epsserr,timerun,epssu]=data_framef2(u,bvec3,bvec2,bvec4,H,mu1,lambda1,lambda2,ND,NPROJ,DataTruth,maxit,maxitcg)
%=======================================================================================
% INPUT:
 % u: initialized image reconstructed by wavelet frame based model
 % bvec3: initialized interpolated signal $f$
 % 

% OUTPUT:
 % DataRes: reconstructed CT image by SRD_DDTF model
 % epsserr: an array of error between restored image and ground truth in each iteration
 % timerun: time consuming at each iteration
 % epssu: an array of error between consecutive restored images in each iteration
%=======================================================================================


%% parameter setting
tic
addpath('./tool/');
[rowu,colu] = size(u);
[rowf,colf] = size(bvec3);
RR = zeros((rowf*colf),1);
I = [];
for i = 1:NPROJ
    I = [I,(2*i-1)*ND+1:2*i*ND];
end
mmu = 65*65*lambda2*2;
mmmu = 51*51*lambda1*2;
epssu = [];
a = mmu;
b = mmmu;

% proximal regulation weights
c1 = mmu*0.001; c2 = c1;
d1 = mmmu*0.001; d2 = d1;

RR(I) = ones((rowf*colf/2),1);
RC = ones((rowf*colf),1) - RR;
finv = 1./(RC+(mmu+a).*ones((rowf*colf),1)+mu1.*RR);
f0 = zeros(rowf*colf,1);
f0(I) = reshape(bvec2,ND*NPROJ,1);

% initialize kernel for $f$
filterrow = 8;
filtercol = 2;
HH1 = dctmtx(filterrow);
HH2 = dctmtx(filtercol);
dict = kron(HH1,HH2)';

% initialize kernel for $u$
filterrow2 = 8;
filtercol2 = 8;
HH1 = dctmtx(filterrow2);
HH2 = dctmtx(filtercol2);
dict2 = kron(HH1,HH2)';


stepSize = 1;
it = 1; 
f = bvec3(:);

%% initialization
Data = im2colstep(bvec3, [filterrow, filtercol], [stepSize, stepSize]);
if NPROJ <= 20
    pf = 50;
    pu = 100;
elseif NPROJ <= 40       
    pf = 100;
    pu = 150;
elseif NPROJ < 60
    pf = 120;
    pu = 200;
else
    pf = 120;
    pu = 300;
end;
    
   % initialize f 
        vb = dict'*Data;
    for i = 1:pf     
       [U,~,V] = svd(Data*vb');
       dict = U*V';
       vb = wthresh(dict'*Data, 'h', sqrt(2*lambda2/mmu));
    end;
    
    % initialize u
    Data2 = im2colstep(u, [filterrow2, filtercol2], [stepSize, stepSize]);
    vb2 = dict2'*Data2;
    for i = 1:pu
        [U,~,V] = svd(Data2*vb2');
        dict2 = U*V'; 
        vb2 = wthresh(dict2'*Data2, 'h', sqrt(2*lambda1/mmmu));
    end;
 
epsserr = [epss(DataTruth,u)];
    
% v, adaptive tight frame coefficients for $f$
% v2, adaptive tight frame coefficients for $u$
    v = wthresh(swt(dict,bvec3,filterrow,filtercol), 'h', sqrt(2*lambda2/mmu));
    v2 = wthresh(swt(dict2,u,filterrow2,filtercol2), 'h', sqrt(2*lambda1/mmmu));
     
tolu = 0.001;
erru = 100;
timerun = [toc];

while (erru > tolu && it < maxit)
    tic;
    fprintf(1,'iteration %d, the epss is %f\n',it,epss(DataTruth,u));
    it = it+1;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % find optimal f
    disp(epss(bvec4,bvec3))
    uk = u(:);
    f = finv.*(RC.*(H*uk)+mu1.*f0+mmu.*reshape(iswt(dict,v,rowf,colf,filterrow,filtercol),rowf*colf,1)+a.*f);
    bvec3 = reshape(f,rowf,colf);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % find optimal u    
    x = uk;
    r = H'*(RC.*f+f0-H*uk)+mmmu.*(reshape(iswt(dict2,v2,rowu,colu,filterrow2,filtercol2),rowu*colu,1)-uk);  
       % % CG
       i = 0;
       while (norm(r) > 1e-10 && i < maxitcg)
           i = i+1;
           if i == 1
               p = r;
           else
               beta = r'*r/(rb'*rb);
               p = r+beta.*pb;
           end
           ap = H*p;
       alpha = r'*r/((mmmu+b).*(p'*p)+ap'*ap);
       x = x+alpha.*p;
       rb = r;
       pb = p;
       r = r-alpha.*(H'*ap+(mmmu+b).*p);
       end
    erru = epss(x,uk);
    u = reshape(x,rowu,colu);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % find optimal W(dict):W_1,W_2
    % % W1  
    Data = im2colstep(bvec3, [filterrow, filtercol], [stepSize, stepSize]);
    [U,~,V] = svd(Data*vb'+(c1/mmu)*dict);
    dict = U*V';
    vb = wthresh((mmu/(d1+mmu))*dict'*Data+(d1/(d1+mmu))*vb, 'h', sqrt(2*lambda2/(mmu+d1)));

    % % W2
    Data2 = im2colstep(u, [filterrow2, filtercol2], [stepSize, stepSize]);
    [U,~,V] = svd(Data2*vb2'+(c2/mmmu)*dict2);
    dict2 = U*V';
    vb2 = wthresh((mmmu/(d2+mmmu))*dict2'*Data2+(d2/(d2+mmmu))*vb2, 'h', sqrt(2*lambda1/(mmmu+d2)));
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % find optimal v1,v2
    % % v1
    v = wthresh((mmu/(d1+mmu))*swt(dict,bvec3,filterrow,filtercol)+(d1/(d1+mmu))*v, 'h', sqrt(2*lambda2/(mmu+d1)));
    % % v2
    v2 = wthresh((mmmu/(d2+mmmu))*swt(dict2,u,filterrow2,filtercol2)+(d2/(d2+mmmu))*v2, 'h', sqrt(2*lambda1/(mmmu+d2)));
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
 fprintf(1,'erru=%f\n',erru);
 epssu = [epssu erru];
 epsserr = [epsserr epss(DataTruth,u)]; 
 trun = toc+timerun(end);  timerun = [timerun trun];
end
DataRes = u;

