function [ output ] = projMatrix2D( L1, L2, N, theta, L, ND)
%   compute projection matrix in sparse form: 
%       projMatrix2D( L1, L2, N, theta, L, ND)
%       this function calculate the ray tracing projection matrix in a
%       sparse form with Siddon's algorithm. It works for 2D case only
%   Input:
%       L1    :   distance from isocenter to source, in pixel size
%       L2    :   distance from isocenter to the center of the detector, in
%                 pixel size
%       N     :   dimension of the image
%       theta :   projection angle counting from positive x direction, in rad 
%       L     :   the length of the imager, in unit of pixel size
%       ND    :   the number of detectors on imager
%   Output:
%       output :   sparse representation of the projection matrix
%   Note:
%       in the output, the index of pixels is in 1d order. the pixel is labeled
%       in this way. i = 1:N*N; i = reshape(i,N,N); then the corresponding
%       element is given by value of i

    id = 1 : ND; 
    z = -L/2 + (id-0.5)*L/ND;
    
    output = [];
    for i = 1:ND,
        output = [output; siddon2D( L1, L2, N, theta, z(i) )];
    end


end

